// TsTable.h: TSe[ubp[NX̃C^[tFCX
//
//////////////////////////////////////////////////////////////////////

#pragma once


#include <vector>
#include "MediaData.h"
#include "TsStream.h"
#include "TsDescriptor.h"


using std::vector;


/////////////////////////////////////////////////////////////////////////////
// PSIe[uۉNX
/////////////////////////////////////////////////////////////////////////////

class CPsiTable
{
public:
	CPsiTable();
	CPsiTable(const CPsiTable &Operand);

	CPsiTable & operator = (const CPsiTable &Operand);

	const bool StoreSection(const CPsiSection *pSection, bool *pbUpdate = NULL);

	const WORD GetExtensionNum(void) const;
	const bool GetExtension(const WORD wIndex, WORD *pwExtension) const;
	const bool GetSectionNum(const WORD wIndex, WORD *pwSectionNum) const;
	const CMediaData * GetSectionData(const WORD wIndex = 0U, const BYTE bySectionNo = 0U) const;

	void Reset(void);

protected:
	struct TAG_TABLEITEM
	{
		WORD wTableIdExtension;				// e[uIDg
		WORD wSectionNum;					// ZNV
		BYTE byVersionNo;					// o[Wԍ
		vector<CMediaData> SectionArray;	// ZNVf[^
	};

	vector<TAG_TABLEITEM> m_TableArray;		// e[u
};



/////////////////////////////////////////////////////////////////////////////
// PSIVOe[uۉNX
/////////////////////////////////////////////////////////////////////////////

class CPsiSingleTable : public CTsPidMapTarget
{
public:
	CPsiSingleTable(const bool bTargetSectionExt = true);
	CPsiSingleTable(const CPsiSingleTable &Operand);
	virtual ~CPsiSingleTable();

	CPsiSingleTable & operator = (const CPsiSingleTable &Operand);

// CTsPidMapTarget
	virtual const bool StorePacket(const CTsPacket *pPacket);
	virtual void OnPidUnmapped(const WORD wPID);

// CPsiSingleTable
	void Reset(void);

	const DWORD GetCrcErrorCount(void) const;

	CPsiSection m_CurSection;

protected:
	virtual const bool OnTableUpdate(const CPsiSection *pCurSection, const CPsiSection *pOldSection);

private:
	static void CALLBACK StoreSection(const CPsiSection *pSection, const PVOID pParam);

	CPsiSectionParser m_PsiSectionParser;
	bool m_bTargetSectionExt;
	bool m_bTableUpdated;
};


/////////////////////////////////////////////////////////////////////////////
// PSIe[uZbgۉNX
/////////////////////////////////////////////////////////////////////////////

class CPsiTableSuite
{
public:
	CPsiTableSuite();
	CPsiTableSuite(const CPsiTableSuite &Operand);

	CPsiTableSuite & operator = (const CPsiTableSuite &Operand);

	const bool StorePacket(const CTsPacket *pPacket);

	void SetTargetSectionExt(const bool bTargetExt);
	const bool AddTable(const BYTE byTableID);

	const WORD GetIndexByID(const BYTE byTableID);
	const CPsiTable * GetTable(const WORD wIndex = 0U) const;
	const CMediaData * GetSectionData(const WORD wIndex = 0U, const WORD wSubIndex = 0U, const BYTE bySectionNo = 0U) const;

	const DWORD GetCrcErrorCount(void) const;
	void Reset(void);

protected:
	struct TAG_TABLESET
	{
		BYTE byTableID;						// e[uID
		CPsiTable PsiTable;					// e[u
	};

	static void CALLBACK StoreSection(const CPsiSection *pSection, const PVOID pParam);

	vector<TAG_TABLESET> m_TableSet;		// e[uZbg

	bool m_bTargetSectionExt;
	bool m_bTableUpdated;

private:
	CPsiSectionParser m_PsiSectionParser;
};


/////////////////////////////////////////////////////////////////////////////
// PATe[uۉNX
/////////////////////////////////////////////////////////////////////////////

class CPatTable : public CPsiSingleTable
{
public:
	CPatTable();
	CPatTable(const CPatTable &Operand);

	CPatTable & operator = (const CPatTable &Operand);

// CPatTable
	const WORD GetNitPID(const WORD wIndex = 0U) const;
	const WORD GetNitNum(void) const;

	const WORD GetPmtPID(const WORD wIndex = 0U) const;
	const WORD GetProgramID(const WORD wIndex = 0U) const;
	const WORD GetProgramNum(void) const;

	void Reset(void);

protected:
	struct TAG_PATITEM
	{
		WORD wProgramID;	// ԑgԍID
		WORD wPID;			// NIT or PMTPID
	};

	virtual const bool OnTableUpdate(const CPsiSection *pCurSection, const CPsiSection *pOldSection);

	vector<TAG_PATITEM> m_NitPIDArray;
	vector<TAG_PATITEM> m_PmtPIDArray;
};


/////////////////////////////////////////////////////////////////////////////
// PMTe[uۉNX
/////////////////////////////////////////////////////////////////////////////

class CPmtTable : public CPsiSingleTable
{
public:
	CPmtTable();
	CPmtTable(const CPmtTable &Operand);

	CPmtTable & operator = (const CPmtTable &Operand);

// CPmtTable
	const WORD GetPcrPID(void) const;
	const CCaMethodDesc * GetCaMethodDesc(void) const;
	const WORD GetEsInfoNum(void) const;
	const BYTE GetStreamTypeID(const WORD wIndex) const;
	const WORD GetEsPID(const WORD wIndex) const;
	const CStreamIdDesc * GetStreamIdDesc(const WORD wIndex) const;

	void Reset(void);

protected:
	struct TAG_PMTITEM
	{
		BYTE byStreamTypeID;			// Stream Type ID
		WORD wEsPID;					// Elementary Stream PID
		CStreamIdDesc m_StreamIdDesc;	// Stream ID Descriptor
	};

	virtual const bool OnTableUpdate(const CPsiSection *pCurSection, const CPsiSection *pOldSection);

	WORD m_wPcrPID;						// PCR_PID
	CCaMethodDesc m_CaMethodDesc;		// Conditional Access Method Descriptor

	vector<TAG_PMTITEM> m_EsInfoArray;
};
